@@ -1,28 +1,15 @@
 /*
- *  Copyright (c) 2007 Sven Langkamp <sven.langkamp@gmail.com>
+ *  SPDX-FileCopyrightText: 2007 Sven Langkamp <sven.langkamp@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_painter_test.h"
-#include <QTest>
-
+#include <simpletest.h>
 
 #include <kis_debug.h>
 #include <QRect>
-#include <QTime>
+#include <QElapsedTimer>
 #include <QtXml>
 
 #include <KoChannelInfo.h>
@@ -37,8 +24,9 @@
 #include "kis_pixel_selection.h"
 #include "kis_fill_painter.h"
 #include <kis_fixed_paint_device.h>
-#include "testutil.h"
+#include <testutil.h>
 #include <kis_iterator_ng.h>
+#include <testimage.h>
 
 void KisPainterTest::allCsApplicator(void (KisPainterTest::* funcPtr)(const KoColorSpace*cs))
 {
@@ -435,7 +423,7 @@ void KisPainterTest::checkPerformance()
     sel->pixelSelection()->select(QRect(0, 0, 10000, 5000), 128);
     sel->updateProjection();
 
-    QTime t;
+    QElapsedTimer t;
     t.start();
     for (int i = 0; i < 10; ++i) {
         KisPainter gc(dst);
@@ -538,7 +526,7 @@ void testMassiveBltFixedImpl(int numRects, bool varyOpacity = false, bool useSel
                                       QString("full_update_%1%2%3")
                                           .arg(numRects)
                                           .arg(opacityPostfix)
-                                          .arg(selectionPostfix), 1, 1));
+                                          .arg(selectionPostfix), 2, 2));
     }
 
     dst->clear();
@@ -560,7 +548,7 @@ void testMassiveBltFixedImpl(int numRects, bool varyOpacity = false, bool useSel
                                       QString("partial_update_%1%2%3")
                                           .arg(numRects)
                                           .arg(opacityPostfix)
-                                          .arg(selectionPostfix), 1, 1));
+                                          .arg(selectionPostfix), 2, 2));
 
     }
 }
